# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""
This plugin provides customization of the header displayed by pytest for
reporting purposes.

PYTEST_DONT_REWRITE

"""
import datetime
import importlib
import locale
import os
import sys
from collections import OrderedDict

TESTED_VERSIONS = {}
PYTEST_HEADER_MODULES = OrderedDict([('Numpy', 'numpy'),
                                    ('Scipy', 'scipy'),
                                    ('Matplotlib', 'matplotlib'),
                                    ('h5py', 'h5py'),
                                    ('Pandas', 'pandas')])


def pytest_addoption(parser):

    group = parser.getgroup("astropy header options")
    group.addoption('--astropy-header', action='store_true',
                    help="Show the pytest-astropy header")
    group.addoption('--astropy-header-packages', default=None,
                    help="Comma-separated list of packages to include in the header")
    parser.addini('astropy_header', type="bool",
                  help="Show the pytest-astropy header")
    parser.addini('astropy_header_packages', type='linelist',
                  help="Comma-separated list of packages to include in the header")


def pytest_report_header(config):

    if not config.getoption("astropy_header") and not config.getini("astropy_header"):
        return

    astropy_header_packages_option = config.getoption("astropy_header_packages")
    astropy_header_packages_ini = config.getini("astropy_header_packages")

    if astropy_header_packages_option is not None:
        if isinstance(astropy_header_packages_option, str):
            astropy_header_packages_option = [x.strip() for x in astropy_header_packages_option.split(',')]
        packages_to_display = OrderedDict([(x, x) for x in astropy_header_packages_option])
    elif len(astropy_header_packages_ini) > 0:
        if len(astropy_header_packages_ini) == 1:
            astropy_header_packages_ini = [x.strip() for x in astropy_header_packages_ini[0].split(',')]
        packages_to_display = OrderedDict([(x, x) for x in astropy_header_packages_ini])
    else:
        packages_to_display = PYTEST_HEADER_MODULES

    try:
        stdoutencoding = sys.stdout.encoding or 'ascii'
    except AttributeError:
        stdoutencoding = 'ascii'

    args = config.args

    s = ''
    for pkg, version in TESTED_VERSIONS.items():
        s += f"\nRunning tests with {pkg} version {version}.\n"

    # Per https://github.com/astropy/astropy/pull/4204, strip the rootdir from
    # each directory argument
    if hasattr(config, 'rootdir'):
        rootdir = str(config.rootdir)
        if not rootdir.endswith(os.sep):
            rootdir += os.sep

        dirs = [arg[len(rootdir):] if arg.startswith(rootdir) else arg
                for arg in args]
    else:
        dirs = args

    s += "Running tests in {}.\n\n".format(" ".join(dirs))

    s += "Date: {}\n\n".format(datetime.datetime.now().isoformat()[:19])

    import warnings
    from platform import platform
    plat = platform()
    if isinstance(plat, bytes):
        plat = plat.decode(stdoutencoding, 'replace')
    s += f"Platform: {plat}\n\n"
    s += f"Executable: {sys.executable}\n\n"
    s += f"Full Python Version: \n{sys.version}\n\n"

    s += "encodings: sys: {}, locale: {}, filesystem: {}".format(
        sys.getdefaultencoding(),
        locale.getpreferredencoding(),
        sys.getfilesystemencoding())
    s += '\n'

    s += f"byteorder: {sys.byteorder}\n"
    s += "float info: dig: {0.dig}, mant_dig: {0.dig}\n\n".format(
        sys.float_info)

    s += "Package versions: \n"

    for module_display, module_name in packages_to_display.items():
        try:
            with warnings.catch_warnings():
                warnings.filterwarnings('ignore', category=DeprecationWarning)
                module = importlib.import_module(module_name)
        except ImportError:
            s += f"{module_display}: not available\n"
        else:
            try:
                version = module.__version__
            except AttributeError:
                version = 'unknown (no __version__ attribute)'
            s += f"{module_display}: {version}\n"

    s += "\n"

    special_opts = ["remote_data", "pep8"]
    opts = []
    for op in special_opts:
        op_value = getattr(config.option, op, None)
        if op_value:
            if isinstance(op_value, str):
                op = ': '.join((op, op_value))
            opts.append(op)
    if opts:
        s += "Using Astropy options: {}.\n".format(", ".join(opts))

    return s



import pytest
import numpy

NUMPY_VERSION = numpy.__version__

pytest_plugins = ['pytester']


def extract_package_version_lines(output):
    lines = []
    in_section = False
    for line in output.splitlines():
        if line.strip() == 'Package versions:':
            in_section = True
        elif in_section:
            if line.strip() == "":
                break
            else:
                lines.append(line)
    return lines


def test_default(testdir, capsys):
    testdir.inline_run()
    out, err = capsys.readouterr()
    assert 'Package versions:' not in out


@pytest.mark.parametrize('method', ['cli', 'ini', 'conftest'])
def test_enabled(testdir, capsys, method):
    if method == 'cli':
        testdir.inline_run("--astropy-header")
    elif method == 'ini':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
        """)
        testdir.inline_run()
    elif method == 'conftest':
        testdir.makeconftest("""
            def pytest_configure(config):
                config.option.astropy_header = True
        """)
        testdir.inline_run()
    out, err = capsys.readouterr()
    lines = extract_package_version_lines(out)
    assert len(lines) == 5
    assert lines[0].startswith('Numpy: ')
    assert lines[1].startswith('Scipy: ')
    assert lines[2].startswith('Matplotlib: ')
    assert lines[3].startswith('h5py: ')
    assert lines[4].startswith('Pandas: ')


@pytest.mark.parametrize('method', ['ini', 'conftest'])
def test_explicit_disable(testdir, capsys, method):
    if method == 'ini':
        testdir.makeini("""
            [pytest]
            astropy_header = no
        """)
        testdir.inline_run()
    elif method == 'conftest':
        testdir.makeconftest("""
            def pytest_configure(config):
                config.option.astropy_header = False
        """)
    testdir.inline_run()
    out, err = capsys.readouterr()
    assert 'Package versions:' not in out


@pytest.mark.parametrize('method', ['cli', 'ini', 'ini_list', 'conftest'])
def test_override_package_single(testdir, capsys, method):
    if method == 'cli':
        testdir.inline_run("--astropy-header", "--astropy-header-packages=numpy")
    elif method == 'ini':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
            astropy_header_packages = numpy
        """)
        testdir.inline_run()
    elif method == 'ini_list':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
            astropy_header_packages =
                numpy
        """)
        testdir.inline_run()
    elif method == 'conftest':
        testdir.makeconftest("""
            def pytest_configure(config):
                config.option.astropy_header = True
                config.option.astropy_header_packages = ['numpy']
        """)
        testdir.inline_run()
    out, err = capsys.readouterr()
    lines = extract_package_version_lines(out)
    assert len(lines) == 1
    assert lines[0] == f'numpy: {NUMPY_VERSION}'


@pytest.mark.parametrize('method', ['cli', 'ini', 'ini_list', 'conftest'])
def test_override_package_multiple(testdir, capsys, method):
    if method == 'cli':
        testdir.inline_run("--astropy-header", "--astropy-header-packages=numpy,pandas")
    elif method == 'ini':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
            astropy_header_packages = numpy, pandas
        """)
        testdir.inline_run()
    elif method == 'ini_list':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
            astropy_header_packages =
                numpy
                pandas
        """)
        testdir.inline_run()
    elif method == 'conftest':
        testdir.makeconftest("""
            def pytest_configure(config):
                config.option.astropy_header = True
                config.option.astropy_header_packages = ['numpy', 'pandas']
        """)
        testdir.inline_run()
    out, err = capsys.readouterr()
    print(out)
    lines = extract_package_version_lines(out)
    assert len(lines) == 2
    assert lines[0] == f'numpy: {NUMPY_VERSION}'
    assert lines[1].startswith('pandas')


@pytest.mark.parametrize('method', ['cli', 'ini', 'ini_list', 'conftest'])
def test_nonexistent(testdir, capsys, method):
    if method == 'cli':
        testdir.inline_run("--astropy-header", "--astropy-header-packages=apackagethatdoesnotexist")
    elif method == 'ini':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
            astropy_header_packages = apackagethatdoesnotexist
        """)
        testdir.inline_run()
    elif method == 'ini_list':
        testdir.makeini("""
            [pytest]
            astropy_header = yes
            astropy_header_packages =
                apackagethatdoesnotexist
        """)
        testdir.inline_run()
    elif method == 'conftest':
        testdir.makeconftest("""
            def pytest_configure(config):
                config.option.astropy_header = True
                config.option.astropy_header_packages = ['apackagethatdoesnotexist']
        """)
        testdir.inline_run()
    out, err = capsys.readouterr()
    lines = extract_package_version_lines(out)
    assert len(lines) == 1
    assert lines[0] == 'apackagethatdoesnotexist: not available'


def test_modify_in_conftest(testdir, capsys):
    testdir.makeconftest("""
    from pytest_astropy_header.display import PYTEST_HEADER_MODULES, TESTED_VERSIONS

    def pytest_configure(config):
        config.option.astropy_header = True
        PYTEST_HEADER_MODULES.pop('Pandas')
        PYTEST_HEADER_MODULES['scikit-image'] = 'skimage'
        TESTED_VERSIONS['fakepackage'] = '1.0.2'
    """)
    testdir.inline_run()
    out, err = capsys.readouterr()
    assert err == ''
    lines = extract_package_version_lines(out)
    assert len(lines) == 5
    assert lines[0].startswith('Numpy: ')
    assert lines[1].startswith('Scipy: ')
    assert lines[2].startswith('Matplotlib: ')
    assert lines[3].startswith('h5py: ')
    assert lines[4].startswith('scikit-image: ')
    assert 'Running tests with fakepackage version 1.0.2' in out
